#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.url import FC3_Url

class Url_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = FC3_Url()
        data2 = FC3_Url()

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['url']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class FC3_TestCase(CommandTest):
    command = "url"

    def runTest(self):
        # pass
        self.assert_parse("url --url=http://domain.com", "url --url=\"http://domain.com\"\n")

        self.assertFalse(self.assert_parse("url --url=http://domain.com") is None)
        self.assertTrue(self.assert_parse("url --url=http://domainA.com") !=
                        self.assert_parse("url --url=http://domainB.com"))
        self.assertFalse(self.assert_parse("url --url=http://domainA.com") ==
                         self.assert_parse("url --url=http://domainB.com"))

        # fail
        # missing required option --url
        self.assert_parse_error("url")
        self.assert_parse_error("url --url")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.seen = False
        self.assertEqual(cmd.__str__(), "")

class F13_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=http://someplace/somewhere --proxy=http://wherever/other",
                          "url --url=\"http://someplace/somewhere\" --proxy=\"http://wherever/other\"\n")

        self.assertTrue(self.assert_parse("url --url=http://domain.com --proxy=http://proxy.com") ==
                        self.assert_parse("url --url=http://domain.com --proxy=http://proxy.com"))
        self.assertFalse(self.assert_parse("url --url=http://domain.com --proxy=http://proxyA.com") ==
                         self.assert_parse("url --url=http://domain.com --proxy=http://proxyB.com"))

        # fail
        self.assert_parse_error("cdrom --proxy=http://someplace/somewhere")
        self.assert_parse_error("url --url=http://someplace/somewhere --proxy")
        self.assert_parse_error("url --proxy=http://someplace/somewhere")

class F14_TestCase(F13_TestCase):
    def runTest(self):
        # run FC6 test case
        F13_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=https://someplace/somewhere --noverifyssl",
                          "url --url=\"https://someplace/somewhere\" --noverifyssl\n")

        self.assertTrue(self.assert_parse("url --url=https://domain.com --noverifyssl") ==
                        self.assert_parse("url --url=https://domain.com --noverifyssl"))
        self.assertFalse(self.assert_parse("url --url=https://domain.com") ==
                         self.assert_parse("url --url=https://domain.com --noverifyssl"))

        # fail
        self.assert_parse_error("cdrom --noverifyssl")

class F18_TestCase(F14_TestCase):
    def runTest(self):
        # run F14 test case.
        F14_TestCase.runTest(self)

        # pass
        self.assert_parse("url --mirrorlist=http://www.wherever.com/mirror",
                          "url --mirrorlist=\"http://www.wherever.com/mirror\"\n")

        self.assertTrue(self.assert_parse("url --mirrorlist=https://domain.com") ==
                        self.assert_parse("url --mirrorlist=https://domain.com"))
        self.assertFalse(self.assert_parse("url --url=https://domain.com") ==
                         self.assert_parse("url --mirrorlist=https://domain.com"))

        # fail
        # missing one of required options --url or --mirrorlist
        self.assert_parse_error("url")
        self.assert_parse_error("url --mirrorlist")

        # It's --url, not --baseurl.
        self.assert_parse_error("url --baseurl=www.wherever.com")

        # only one of --url or --mirrorlist may be specified
        self.assert_parse_error("url --url=www.wherever.com --mirrorlist=www.wherever.com")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.seen = True
        cmd.url = None
        cmd.mirrorlist = None
        self.assertEqual(cmd.__str__(), "# Use network installation\n\n")

class F27_TestCase(F18_TestCase):
    def runTest(self):
        # run F18 test case.
        F18_TestCase.runTest(self)

        # pass
        self.assert_parse("url --metalink=http://www.wherever.com/metalink",
                          "url --metalink=\"http://www.wherever.com/metalink\"\n")

        self.assertTrue(self.assert_parse("url --metalink=https://domain.com") == \
                        self.assert_parse("url --metalink=https://domain.com"))
        self.assertFalse(self.assert_parse("url --url=https://domain.com") == \
                         self.assert_parse("url --metalink=https://domain.com"))

        # fail
        self.assert_parse_error("url --metalink")

        # only one of --url, --mirrorlist, or --metalink may be specified
        self.assert_parse_error("url --url=www.wherever.com --metalink=www.wherever.com")
        self.assert_parse_error("url --mirrorlist=www.wherever.com --metalink=www.wherever.com")

class F30_TestCase(F27_TestCase):
    def runTest(self):
        # run F27 test case.
        F27_TestCase.runTest(self)

        # pass
        self.assert_parse("url --url=http://example.com --sslclientcert=file:///foo/bar",
                          "url --url=\"http://example.com\" --sslclientcert=\"file:///foo/bar\"\n")
        self.assert_parse("url --url=http://example.com --sslclientkey=file:///foo/bar",
                          "url --url=\"http://example.com\" --sslclientkey=\"file:///foo/bar\"\n")
        self.assert_parse("url --url=http://example.com --sslcacert=file:///foo/bar",
                          "url --url=\"http://example.com\" --sslcacert=\"file:///foo/bar\"\n")

        # fail: all of these take arguments
        self.assert_parse_error("url --url=http://example.com --sslclientcert")
        self.assert_parse_error("url --url=http://example.com --sslclientkey")
        self.assert_parse_error("url --url=http://example.com --sslcacert")

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest
from pykickstart.errors import KickstartDeprecationWarning
from pykickstart.base import RemovedCommand

class FC3_TestCase(CommandTest):
    command = "bootloader"

    def runTest(self, iscrypted=False):
        if "--linear" in self.optionList:
            linear = "--linear "
        else:
            linear = ""

        # pass
        self.assert_parse("bootloader --append=rhgb", "bootloader --append=\"rhgb\" %s--location=mbr\n" % linear)
        self.assert_parse("bootloader --append=\"rhgb quiet\"", "bootloader --append=\"rhgb quiet\" %s--location=mbr\n" % linear)
        self.assert_parse("bootloader", "bootloader %s--location=mbr\n" % linear)

        if "--linear" in self.optionList and "--nolinear" in self.optionList:
            self.assert_parse("bootloader --nolinear", "bootloader --location=mbr\n")
            self.assert_parse("bootloader --nolinear --linear", "bootloader --linear --location=mbr\n")
            self.assert_parse("bootloader --linear --nolinear", "bootloader --location=mbr\n")

        for loc in ["mbr", "partition", "none", "boot"]:
            self.assert_parse("bootloader --location=%s" % loc, "bootloader %s--location=%s\n" % (linear, loc))

        if "--lba32" in self.optionList:
            self.assert_parse("bootloader --lba32", "bootloader %s--location=mbr --lba32\n" % linear)

        self.assert_parse("bootloader --password=blahblah", "bootloader %s--location=mbr --password=\"blahblah\"\n" % linear)
        if not iscrypted:
            self.assert_parse("bootloader --md5pass=blahblah", "bootloader %s--location=mbr --md5pass=\"blahblah\"\n" % linear)
        if "--upgrade" in self.optionList:
            self.assert_parse("bootloader --upgrade", "bootloader %s--location=mbr --upgrade\n" % linear)
        self.assert_parse("bootloader --driveorder=hda,sdb", "bootloader %s--location=mbr --driveorder=\"hda,sdb\"\n" % linear)

        if "--useLilo" in self.optionList:
            self.assert_parse("bootloader --useLilo", "bootloader %s--location=mbr --useLilo\n" % linear)
            self.assert_parse("lilo")

        # fail
        self.assert_parse_error("bootloader --append")
        self.assert_parse_error("bootloader --location=nowhere")
        self.assert_parse_error("bootloader --password")
        self.assert_parse_error("bootloader --md5pass")
        self.assert_parse_error("bootloader --driveorder")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.location = ""
        cmd.linear = False
        self.assertEqual(cmd.__str__(), "")
        self.assertEqual(cmd._getArgsAsStr(), "")

class FC4_TestCase(FC3_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        FC3_TestCase.runTest(self, iscrypted)

        # Ensure these options have been removed.
        self.assert_removed("bootloader", "--linear")
        self.assert_removed("bootloader", "--nolinear")
        self.assert_removed("bootloader", "--useLilo")

class FC4_Lilo_TestCase(CommandTest):
    def runTest(self):
        # make sure that lilo is removed
        cmd = self.handler().commands["lilo"]
        self.assertTrue(issubclass(cmd.__class__, RemovedCommand))

class F8_TestCase(FC4_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        FC4_TestCase.runTest(self, iscrypted)

        # pass
        self.assert_parse("bootloader --timeout 47", "bootloader --location=mbr --timeout=47\n")
        self.assert_parse("bootloader --default=this", "bootloader --location=mbr --default=this\n")

        # fail
        self.assert_parse_error("bootloader --timeout")
        self.assert_parse_error("bootloader --default")

class F12_TestCase(F8_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        F8_TestCase.runTest(self, iscrypted)

        # deprecated
        self.assert_deprecated("bootloader", "--lba32")

class F14_TestCase(F12_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        F12_TestCase.runTest(self, iscrypted)

        # fail
        self.assert_parse_error("bootloader --lba32")

class F15_TestCase(F14_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        F14_TestCase.runTest(self, iscrypted=True)

        # pass
        self.assert_parse("bootloader --password=blahblah --iscrypted", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
        self.assert_parse("bootloader --md5pass=blahblah", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")

class F17_TestCase(F15_TestCase):
    def runTest(self, iscrypted=False):
        # run parent tests
        F15_TestCase.runTest(self, iscrypted=iscrypted)

        self.assert_parse("bootloader --location=mbr --boot-drive=/dev/sda",
                          "bootloader --location=mbr --boot-drive=/dev/sda\n")
        self.assert_parse("bootloader --location=mbr --boot-drive=sda",
                          "bootloader --location=mbr --boot-drive=sda\n")
        self.assert_parse("bootloader --location=mbr --boot-drive=/dev/disk/by-path/pci-0000:00:0e.0-scsi-0:0:0:0",
                          "bootloader --location=mbr --boot-drive=/dev/disk/by-path/pci-0000:00:0e.0-scsi-0:0:0:0\n")

        self.assert_parse_error("bootloader --location=mbr --boot-drive=sda,sdb")

class F18_TestCase(F17_TestCase):
    def runTest(self, iscrypted=False):
        # run parent tests
        F17_TestCase.runTest(self, iscrypted=iscrypted)

        self.assert_parse("bootloader --location=mbr --timeout=5 --append=\"rhgb quiet\"")
        self.assert_parse("bootloader --location=mbr --timeout=5 --leavebootorder --append=\"rhgb quiet\"",
                          "bootloader --append=\"rhgb quiet\" --location=mbr --timeout=5 --leavebootorder\n")

class RHEL5_TestCase(FC4_TestCase):
    def runTest(self, iscrypted=False):
        FC4_TestCase.runTest(self, iscrypted)

        self.assert_parse("bootloader --hvargs=bleh",
                          "bootloader --location=mbr --hvargs=\"bleh\"\n")
        self.assert_parse("bootloader --hvargs=\"bleh bleh\"",
                          "bootloader --location=mbr --hvargs=\"bleh bleh\"\n")
        self.assert_parse_error("bootloader --hvargs")

class RHEL6_TestCase(F12_TestCase):
    def runTest(self, iscrypted=False):
        # Run parent tests
        F12_TestCase.runTest(self, iscrypted=True)

        # pass
        self.assert_parse("bootloader --password=blahblah --iscrypted", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")
        self.assert_parse("bootloader --md5pass=blahblah", "bootloader --location=mbr --password=\"blahblah\" --iscrypted\n")

class F19_TestCase(F18_TestCase):
    def runTest(self, iscrypted=False):
        # run parent tests
        F18_TestCase.runTest(self, iscrypted=iscrypted)

        self.assert_parse("bootloader --location=mbr --timeout=5 --append=\"rhgb quiet\"")
        self.assert_parse("bootloader --location=mbr --timeout=5 --extlinux --append=\"rhgb quiet\"",
                          "bootloader --append=\"rhgb quiet\" --location=mbr --timeout=5 --extlinux\n")

class F21_TestCase(F19_TestCase):
    def runTest(self, iscrypted=False):
        # run parent tests
        F19_TestCase.runTest(self, iscrypted=iscrypted)

        self.assert_parse("bootloader --disabled", "bootloader --disabled\n")
        self.assert_parse("bootloader --location=mbr --disabled", "bootloader --disabled\n")
        self.assert_parse("bootloader --location=mbr --nombr", "bootloader --location=mbr --nombr\n")

class RHEL7_TestCase(F21_TestCase):
    def runTest(self, iscrypted=False):
        F21_TestCase.runTest(self, iscrypted=iscrypted)

class F29_TestCase(F21_TestCase):
    def runTest(self, iscrypted=False):
        F21_TestCase.runTest(self, iscrypted=iscrypted)

        if "--upgrade" in self.optionList:
            self.assert_deprecated("bootloader", "--upgrade")

            # deprecated options should also raise a deprecation warning - test that somewhere
            with self.assertWarns(KickstartDeprecationWarning):
                self.assert_parse("bootloader --upgrade")

class RHEL8_TestCase(F29_TestCase):
    pass

class F34_TestCase(F29_TestCase):
    def runTest(self, iscrypted=False):
        F29_TestCase.runTest(self, iscrypted=iscrypted)
        self.assert_removed("bootloader", "--upgrade")

if __name__ == "__main__":
    unittest.main()



#
# errors.py:  Kickstart error handling.
#
# Chris Lumens <clumens@redhat.com>
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Error and warning handling classes and functions.

This module exports several exception classes:

    KickstartError - A generic exception class.

    KickstartParseError - An exception for errors occurring during parsing.

    KickstartValueError - No longer raised by pykickstart, but kept around for
                          backwards compatibility.

    KickstartVersionError - An exception for errors relating to unsupported
                            syntax versions.

And some warning classes:

    KickstartWarning - A generic warning class.

    KickstartParseWarning - A class for warnings occurring during parsing.

    KickstartDeprecationWarning - A class for warnings occurring during parsing
                                  related to deprecated commands and options.

"""
import warnings
from pykickstart.i18n import _


def formatErrorMsg(lineno, msg=""):
    """This function is deprecated. KickstartError formats the error message now,
       so this function returns a tuple that can be formatted by KickstartError.

       You should call:
       KickstartError(message, lineno=lineno)

       But the deprecated way is still supported:
       KickstartError(formatErrorMsg(message, lineno=lineno))

    """
    warnings.warn("Function formatErrorMsg is deprecated. The error messages "
                  "are formatted by KickstartError now.", DeprecationWarning)

    return lineno, msg

def _format_error_message(lineno, msg=""):
    """Properly format the error message msg in an exception.
       This function should be called only in exceptions to format the error messages.
    """
    if msg:
        return _("The following problem occurred on line %(lineno)s of the kickstart file:"
                 "\n\n%(msg)s\n") % {"lineno": lineno, "msg": msg}

    return _("There was a problem reading from line %s of the kickstart file") % lineno

class KickstartError(Exception):
    """A generic exception class for unspecific error conditions."""

    def __init__(self, msg="", lineno=None, formatting=True):
        """Create a new KickstartError exception instance with the descriptive
           message msg. The msg will be formatted if formatting is allowed and
           the line number lineno is set.
        """
        Exception.__init__(self)
        self.message = msg
        self.lineno = lineno

        # Accept tuples from formatErrorMsg for backwards compatibility.
        if isinstance(msg, tuple) and len(msg) == 2:
            self.lineno, self.message = msg

        # Keep the value attribute for backwards compatibility.
        self.value = self.message

        # Format the error message if it is allowed.
        if formatting and self.lineno is not None:
            self.value = _format_error_message(self.lineno, self.message)

    def __str__(self):
        return self.value

class KickstartParseError(KickstartError):
    """An exception class for errors when processing the input file, such as
       unknown options, commands, or sections.
    """

class KickstartValueError(KickstartError):
    """This exception class is no longer raised by pykickstart but is kept
       for backwards compatibility.
    """

class KickstartVersionError(KickstartError):
    """An exception class for errors related to using an incorrect version of
       kickstart syntax.
    """

class KickstartWarning(Warning):
    """A generic warning class for unspecific conditions."""

class KickstartParseWarning(KickstartWarning, UserWarning):
    """A class for warnings occurring during parsing an input file, such as
       defining duplicate entries and setting removed keywords.
    """

class KickstartDeprecationWarning(KickstartParseWarning, DeprecationWarning):
    """A class for warnings occurring during parsing related to using deprecated
       commands and options.
    """



# Andy Lindeberg <alindebe@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.logging import FC6_Logging
from pykickstart.errors import KickstartDeprecationWarning


class FC6_TestCase(CommandTest):
    command = "logging"

    def runTest(self):
        cmd = FC6_Logging()
        self.assertEqual(cmd.__str__(), '\n')
        self.assertEqual(cmd.level, "info")

        # pass
        self.assert_parse("logging", "logging --level=info\n")
        self.assert_parse("logging --level=debug", "logging --level=debug\n")
        self.assert_parse("logging --level=info", "logging --level=info\n")
        self.assert_parse("logging --level=warning", "logging --level=warning\n")
        self.assert_parse("logging --level=error", "logging --level=error\n")
        self.assert_parse("logging --level=critical", "logging --level=critical\n")
        self.assert_parse("logging --host=HOSTNAME", "logging --level=info --host=HOSTNAME\n")
        self.assert_parse("logging --host=HOSTNAME --port=PORT", "logging --level=info --host=HOSTNAME --port=PORT\n")

        # fail
        self.assert_parse_error("logging --level")
        self.assert_parse_error("logging --level=''")
        self.assert_parse_error("logging --level=theprincessisinanothercastle")
        self.assert_parse_error("logging --host")
        self.assert_parse_error("logging --port")
        self.assert_parse_error("logging --port=PORT")


class F34_TestCase(CommandTest):
    command = "logging"

    def runTest(self):
        # pass
        self.assert_parse("logging", "")
        self.assert_parse("logging --host=HOSTNAME", "logging --host=HOSTNAME\n")
        self.assert_parse("logging --host=HOSTNAME --port=PORT", "logging --host=HOSTNAME --port=PORT\n")

        # fail
        self.assert_parse_error("logging --host")
        self.assert_parse_error("logging --port")
        self.assert_parse_error("logging --port=PORT")

        # deprecated
        self.assert_deprecated("logging", "--level")

        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("logging --level=info", "")

        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("logging --level=info --host=HOSTNAME", "logging --host=HOSTNAME\n")

        with self.assertWarns(KickstartDeprecationWarning):
            self.assert_parse("logging --level=info --host=HOSTNAME --port=PORT", "logging --host=HOSTNAME --port=PORT\n")


if __name__ == "__main__":
    unittest.main()



import os
import six
import tempfile
from tests.baseclass import ParserTest

from pykickstart.errors import KickstartError, KickstartParseError
from pykickstart.parser import preprocessKickstart, preprocessFromString, preprocessKickstartToString, preprocessFromStringToString

###
### TESTING preprocessKickstart
###

class PK_No_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

    def setUp(self):
        ParserTest.setUp(self)
        self._processedPath = None

        (handle, self._path) = tempfile.mkstemp(prefix="ks-", text=True)
        s = self.ks
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._path)
        if self._processedPath:
            os.unlink(self._processedPath)

    def runTest(self):
        self._processedPath = preprocessKickstart(self._path)
        with open(self._processedPath) as f:
            self.assertEqual(f.read(), self.ks)

class PK_Ksappend_Missing(PK_No_Ksappend):
    def __init__(self, *args, **kwargs):
        PK_No_Ksappend.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
%ksappend /tmp/MISSING_FILE
"""

    def runTest(self):
        self.assertRaises(KickstartError, preprocessKickstart, self._path)

class PK_With_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

        self.ksappend = """
timezone America/New_York
"""

    def setUp(self):
        ParserTest.setUp(self)
        self._processedPath = None

        (handle, self._ksappendPath) = tempfile.mkstemp(prefix="ksappend-", text=True)
        s = self.ksappend
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

        # Write the ksappend file first so we know its filename for making the
        # %ksappend line.
        (handle, self._path) = tempfile.mkstemp(prefix="ks-", text=True)
        s = self.ks + "%ksappend " + self._ksappendPath
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._path)
        os.unlink(self._ksappendPath)
        if self._processedPath:
            os.unlink(self._processedPath)

    def runTest(self):
        self._processedPath = preprocessKickstart(self._path)
        with open(self._processedPath) as f:
            self.assertEqual(f.read(), self.ks + self.ksappend)

###
### TESTING preprocessFromString
###

class PFS_No_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

    def setUp(self):
        ParserTest.setUp(self)
        self._path = None

    def runTest(self):
        self._path = preprocessFromString(self.ks)
        with open(self._path) as f:
            self.assertEqual(f.read(), self.ks)

    def tearDown(self):
        ParserTest.tearDown(self)
        if self._path:
            os.unlink(self._path)

class PFS_Ksappend_Missing(PFS_No_Ksappend):
    def __init__(self, *args, **kwargs):
        PFS_No_Ksappend.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
%ksappend /tmp/MISSING_FILE
"""

    def runTest(self):
        self.assertRaises(KickstartError, preprocessFromString, self.ks)

class PFS_Ksappend_Invalid(PFS_No_Ksappend):
    def __init__(self, *args, **kwargs):
        PFS_No_Ksappend.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
%ksappend
"""

    def runTest(self):
        self.assertRaises(KickstartParseError, preprocessFromString, self.ks)

class PFS_With_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

        self.ksappend = """
timezone America/New_York
"""

    def setUp(self):
        ParserTest.setUp(self)
        self._path = None

        (handle, self._ksappendPath) = tempfile.mkstemp(prefix="ksappend-", text=True)
        s = self.ksappend
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._ksappendPath)
        if self._path:
            os.unlink(self._path)

    def runTest(self):
        self._path = preprocessFromString(self.ks + "%ksappend " + self._ksappendPath)
        with open(self._path) as f:
            self.assertEqual(f.read(), self.ks + self.ksappend)

###
### TESTING preprocessKickstartToString
###

class PKTS_No_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

    def setUp(self):
        ParserTest.setUp(self)

        (handle, self._path) = tempfile.mkstemp(prefix="ks-", text=True)
        s = self.ks
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._path)

    def runTest(self):
        processed = preprocessKickstartToString(self._path)
        self.assertEqual(processed.decode(), self.ks)

class PKTS_Ksappend_Missing(PKTS_No_Ksappend):
    def __init__(self, *args, **kwargs):
        PKTS_No_Ksappend.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
%ksappend /tmp/MISSING_FILE
"""

    def runTest(self):
        self.assertRaises(KickstartError, preprocessKickstartToString, self._path)

class PKTS_Kickstart_Missing(ParserTest):
    def runTest(self):
        self.assertRaises(KickstartError, preprocessKickstartToString, "/tmp/MISSING_FILE")

class PKTS_With_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

        self.ksappend = """
timezone America/New_York
"""

    def setUp(self):
        ParserTest.setUp(self)

        (handle, self._ksappendPath) = tempfile.mkstemp(prefix="ksappend-", text=True)
        s = self.ksappend
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

        # Write the ksappend file first so we know its filename for making the
        # %ksappend line.
        (handle, self._path) = tempfile.mkstemp(prefix="ks-", text=True)
        s = self.ks + "%ksappend " + self._ksappendPath
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._path)
        os.unlink(self._ksappendPath)

    def runTest(self):
        processed = preprocessKickstartToString(self._path)
        self.assertEqual(processed.decode(), self.ks + self.ksappend)

###
### TESTING preprocessFromStringToString
###

class PFSTS_No_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

    def runTest(self):
        processed = preprocessFromStringToString(self.ks)
        self.assertEqual(processed.decode(), self.ks)

class PFSTS_Ksappend_Missing(PFSTS_No_Ksappend):
    def __init__(self, *args, **kwargs):
        PFSTS_No_Ksappend.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
%ksappend /tmp/MISSING_FILE
"""

    def runTest(self):
        self.assertRaises(KickstartError, preprocessFromStringToString, self.ks)

class PFSTS_With_Ksappend(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
lang en_US
keyboard us
autopart
"""

        self.ksappend = """
timezone America/New_York
"""

    def setUp(self):
        ParserTest.setUp(self)

        (handle, self._ksappendPath) = tempfile.mkstemp(prefix="ksappend-", text=True)
        s = self.ksappend
        if six.PY3:
            s = s.encode('utf-8')

        os.write(handle, s)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._ksappendPath)

    def runTest(self):
        processed = preprocessFromStringToString(self.ks + "%ksappend " + self._ksappendPath)
        self.assertEqual(processed.decode(), self.ks + self.ksappend)



#
# Martin Kolman <mkolman@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import RHEL8

from pykickstart.i18n import _

class RHEL8_Syspurpose(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.role = kwargs.get("role", None)
        self.sla = kwargs.get("sla", None)
        self.usage = kwargs.get("usage", None)
        self.addons = kwargs.get("addons", None)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if not retval and not any([self.role, self.sla, self.usage, self.addons]):
            return ""

        retval += '# Intended system purpose\nsyspurpose'
        if self.role:
            retval+=' --role="%s"' % self.role
        if self.sla:
            retval+=' --sla="%s"' % self.sla
        if self.usage:
            retval+=' --usage="%s"' % self.usage
        if self.addons:
            for addon in self.addons:
                retval+=' --addon="%s"' % addon
        retval+='\n'
        return retval

    def _getParser(self):
        op = KSOptionParser(prog="syspurpose", description="""
                            The syspurpose command is used to describe
                            how the system is intended to be used after the installation.

                            This information then can be used to apply the
                            correct subscription entitlement to the system.""", version=RHEL8)
        op.add_argument("--role", metavar="<role_name>", version=RHEL8, required=False,
                        help="""
                        The intended role of the system.""")
        op.add_argument("--sla", metavar="<sla_name>", version=RHEL8, required=False,
                        help="""
                        Name of the sla intended for the system.""")
        op.add_argument("--usage", metavar="<usage>", version=RHEL8, required=False,
                        help="""
                        The intended usage of the system.""")
        op.add_argument("--addon", metavar="<layered product or feature>", version=RHEL8, required=False,
                        action="append", dest="addons", help="""
                        Any additional layered products or features. To add multiple items specify
                        --addon multiple times, once per layered product/feature.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) > 0:
            msg = _("The syspurpose command does not take positional arguments!")
            raise KickstartParseError(msg, lineno=self.lineno)

        self.set_to_self(ns)
        return self



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.fcoe import F12_FcoeData
from pykickstart.version import F12

class F12_TestCase(CommandTest):
    def runTest(self):
        data1 = F12_FcoeData()
        data2 = F12_FcoeData()
        self.assertEqual(data1, data2)
        self.assertFalse(data1 != data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['nic']:
            setattr(data1, atr, None)
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, None)
            setattr(data2, atr, None)

        # pass
        self.assert_parse("fcoe --nic=eth0",
                          "fcoe --nic=eth0\n")

        # fail - missing required argument
        self.assert_parse_error("fcoe")
        self.assert_parse_error("fcoe --nic")

        # fail - unknown argument
        self.assert_parse_error("fcoe --bogus-option")

        # equality
        self.assertEqual(self.assert_parse("fcoe --nic=eth0"), self.assert_parse("fcoe --nic=eth0"))
        self.assertNotEqual(self.assert_parse("fcoe --nic=eth0"), None)
        self.assertNotEqual(self.assert_parse("fcoe --nic=eth0"), self.assert_parse("fcoe --nic=eth1"))

        # extra test coverage
        cmd = self.handler().commands["fcoe"]
        cmd.fcoe = "--name=blah"
        self.assertEqual(cmd.__str__(), "--name=blah")

        data = self.handler().FcoeData()
        data.nic = ""
        self.assertEqual(data._getArgsAsStr(), "")

class F12_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F12

    def runTest(self):
        self.assert_parse("""
fcoe --nic=eth0
fcoe --nic=eth1""")

        self.assert_parse_error("""
fcoe --nic=eth0
fcoe --nic=eth0""", KickstartParseWarning)

class F13_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)

        self.assert_parse("fcoe --nic=eth0 --dcb",
                          "fcoe --nic=eth0 --dcb\n")

class RHEL7_TestCase(F13_TestCase):
    def runTest(self):
        F13_TestCase.runTest(self)

        self.assert_parse("fcoe --nic=eth0 --autovlan",
                          "fcoe --nic=eth0 --autovlan\n")

class F28_TestCase(F13_TestCase):
    def runTest(self):
        F13_TestCase.runTest(self)

        self.assert_parse("fcoe --nic=eth0 --autovlan",
                          "fcoe --nic=eth0 --autovlan\n")

if __name__ == "__main__":
    unittest.main()



from tools import ksshell
import unittest.mock as mock
from unittest import TestCase

from pykickstart.version import DEVEL, makeVersion


class InvalidKSVersion_Test(TestCase):
    @mock.patch('tools.ksshell.print')
    def runTest(self, _print):
        retval = ksshell.main(['-v', 'INVALID'])
        self.assertEqual(retval, 1)
        _print.assert_called_with('The version INVALID is not supported by pykickstart')

class KickstartCompleter_Test(TestCase):
    def runTest(self):
        kshandler = makeVersion(DEVEL)
        self.assertIsNotNone(kshandler)

        # Did it add the commands, and is there at least one (part) that should be present?
        ksc = ksshell.KickstartCompleter(kshandler, {})
        self.assertTrue(len(ksc.commands) > 0)
        self.assertIn("part", ksc.commands)

        # Test tab completion on 'part [TAB]'
        # Initialize the matches with a mocked input line
        ksc._init_matches("part ", 5, 5)
        self.assertEqual(ksc.complete("", 1), "--fstype")

        # Test tab completion on 'auth[TAB]'
        ksc._init_matches("auth", 0, 5)

        # Python 3.5 returns things in a different order, just make sure they are there and different
        self.assertIn(ksc.complete("", 1), ["auth", "authconfig", "authselect"])
        self.assertIn(ksc.complete("", 2), ["auth", "authconfig", "authselect"])
        self.assertNotEqual(ksc.complete("", 1), ksc.complete("", 2))



import unittest
from tests.baseclass import ParserTest

from pykickstart import constants
from pykickstart.errors import KickstartParseError
from pykickstart import version         # pylint: disable=unused-import

class Simple_Terminated_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%pre-install
ls /tmp
%end
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify the script defaults.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/bin/sh")
        self.assertFalse(script.inChroot)
        self.assertEqual(script.lineno, 2)
        self.assertFalse(script.errorOnFail)
        self.assertEqual(script.type, constants.KS_SCRIPT_PREINSTALL)

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Simple_Header_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%pre-install --interpreter /usr/bin/python --erroronfail --log=/tmp/blah
ls /tmp
%end
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify the changes we made in the header.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/usr/bin/python")
        self.assertFalse(script.inChroot)
        self.assertTrue(script.errorOnFail)
        self.assertEqual(script.lineno, 2)
        self.assertEqual(script.type, constants.KS_SCRIPT_PREINSTALL)
        self.assertEqual(script.logfile, "/tmp/blah")

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Pre_Given_Nochroot_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%pre-install --nochroot
ls /tmp
%end
"""

    def runTest(self):
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, self.ks)

class Multiple_Terminated_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%pre-install
ls /tmp
%end

%pre-install
ls /var
%end
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertEqual(len(self.handler.scripts), 2)

        # Verify the script defaults.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/bin/sh")
        self.assertFalse(script.inChroot)
        self.assertFalse(script.errorOnFail)
        self.assertEqual(script.lineno, 2)
        self.assertEqual(script.type, constants.KS_SCRIPT_PREINSTALL)

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

        script = self.handler.scripts[1]
        self.assertEqual(script.script.rstrip(), "ls /var")

class Simple_Unterminated_TestCase(Simple_Terminated_TestCase):
    def __init__(self, *args, **kwargs):
        Simple_Terminated_TestCase.__init__(self, *args, **kwargs)
        self.version = version.F7
        self.ks = """
%pre-install
ls /tmp
"""

class Simple_Unterminated_Fails_TestCase(Simple_Unterminated_TestCase):
    def __init__(self, *args, **kwargs):
        Simple_Unterminated_TestCase.__init__(self, *args, **kwargs)
        self.version = version.F8

    def runTest(self):
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, self.ks)

class Multiple_Unterminated_TestCase(Multiple_Terminated_TestCase):
    def __init__(self, *args, **kwargs):
        Multiple_Terminated_TestCase.__init__(self, *args, **kwargs)
        self.version = version.F7
        self.ks = """
%pre-install
ls /tmp

%pre-install
ls /var
"""

class Multiple_Unterminated_Fails_TestCase(Multiple_Unterminated_TestCase):
    def __init__(self, *args, **kwargs):
        Multiple_Unterminated_TestCase.__init__(self, *args, **kwargs)
        self.version = version.F8

    def runTest(self):
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, self.ks)

if __name__ == "__main__":
    unittest.main()



import unittest

from tests.baseclass import ParserTest

from pykickstart.constants import KS_SCRIPT_POST, KS_SCRIPT_PRE, KS_SCRIPT_PREINSTALL, KS_SCRIPT_TRACEBACK, KS_SCRIPT_ONERROR
from pykickstart.parser import Script
from pykickstart.version import F7

class Script_Object_TestCase(ParserTest):
    def runTest(self):
        # The parser object will never be set up by this test, which means the
        # Script object will get the version number of whatever ran last.  That
        # could be totally wrong.  Thus, cause the parser to be created here
        # so the version will be right.
        self.get_parser()

        body = "import sys\nsys.exit(1)\n"
        obj = Script(body, type=KS_SCRIPT_PRE, interp="/usr/bin/python", logfile="/tmp/log", errorOnFail=True)
        self.assertEqual(obj.type, KS_SCRIPT_PRE)
        self.assertEqual(obj.interp, "/usr/bin/python")
        self.assertEqual(obj.logfile, "/tmp/log")
        self.assertTrue(obj.errorOnFail)

        self.assertEqual(str(obj), """
%pre --interpreter=/usr/bin/python --logfile=/tmp/log --erroronfail
import sys
sys.exit(1)
%end
""")

        obj = Script("ls /", type=KS_SCRIPT_POST, inChroot=False)
        self.assertEqual(obj.type, KS_SCRIPT_POST)
        self.assertFalse(obj.inChroot)

        self.assertEqual(str(obj), """
%post --nochroot
ls /
%end
""")

        obj = Script("ls /", type=KS_SCRIPT_PREINSTALL)
        self.assertEqual(obj.type, KS_SCRIPT_PREINSTALL)

        self.assertEqual(str(obj), """
%pre-install
ls /
%end
""")

        obj = Script("ls /", type=KS_SCRIPT_TRACEBACK)
        self.assertEqual(obj.type, KS_SCRIPT_TRACEBACK)

        self.assertEqual(str(obj), """
%traceback
ls /
%end
""")

        obj = Script("ls /", type=KS_SCRIPT_ONERROR)
        self.assertEqual(obj.type, KS_SCRIPT_ONERROR)

        self.assertEqual(str(obj), """
%onerror
ls /
%end
""")

class Old_Script_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.version = F7

    def runTest(self):
        self.get_parser()

        obj = Script("ls /\n", type=KS_SCRIPT_POST)
        self.assertEqual(str(obj), """
%post --nochroot
ls /
""")

        obj = Script("ls /", type=KS_SCRIPT_POST)
        self.assertEqual(str(obj), """
%post --nochroot
ls /
""")

if __name__ == "__main__":
    unittest.main()



import unittest
import warnings

from pykickstart.parser import KickstartParser
from pykickstart.version import makeVersion
from tests.baseclass import ParserTest
from pykickstart.handlers import control
from pykickstart.base import DeprecatedCommand, RemovedCommand
from pykickstart.errors import KickstartParseError, KickstartDeprecationWarning


class Platform_Comment_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
#platform=x86_64
autopart
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertEqual(self.handler.platform, "x86_64")

class WritePriority_TestCase(unittest.TestCase):
    def runTest(self):
        for _version, _map in control.commandMap.items():
            for _name, command_class in _map.items():
                cmd = command_class()
                if issubclass(cmd.__class__, DeprecatedCommand):
                    self.assertEqual(None, cmd.writePriority, command_class)
                elif issubclass(cmd.__class__, RemovedCommand):
                    self.assertEqual(None, cmd.writePriority, command_class)
                elif _name in ['bootloader', 'lilo', 'zipl']:
                    self.assertEqual(10, cmd.writePriority, command_class)
                elif _name in ['multipath']:
                    self.assertEqual(50, cmd.writePriority, command_class)
                elif _name in ['dmraid']:
                    self.assertEqual(60, cmd.writePriority, command_class)
                elif _name in ['iscsiname']:
                    self.assertEqual(70, cmd.writePriority, command_class)
                elif _name in ['fcoe', 'zfcp', 'iscsi']:
                    self.assertEqual(71, cmd.writePriority, command_class)
                elif _name in ['nvdimm']:
                    self.assertEqual(80, cmd.writePriority, command_class)
                elif _name in ['autopart', 'reqpart']:
                    self.assertEqual(100, cmd.writePriority, command_class)
                elif _name in ['zerombr']:
                    self.assertEqual(110, cmd.writePriority, command_class)
                elif _name in ['clearpart']:
                    self.assertEqual(120, cmd.writePriority, command_class)
                elif _name in ['partition', 'part']:
                    self.assertEqual(130, cmd.writePriority, command_class)
                elif _name in ['raid']:
                    self.assertEqual(131, cmd.writePriority, command_class)
                elif _name in ['volgroup', 'btrfs']:
                    self.assertEqual(132, cmd.writePriority, command_class)
                elif _name in ['logvol']:
                    self.assertEqual(133, cmd.writePriority, command_class)
                elif _name in ['snapshot']:
                    self.assertEqual(140, cmd.writePriority, command_class)
                else:
                    self.assertEqual(0, cmd.writePriority, command_class)

class DeprecatedCommandsParsing_TestCase(unittest.TestCase):
    def runTest(self):
        for version, command_map in control.commandMap.items():

            handler = makeVersion(version)
            parser = KickstartParser(handler)

            for command_name, command_class in command_map.items():
                if not issubclass(command_class, DeprecatedCommand):
                    continue
                if issubclass(command_class, RemovedCommand):
                    continue

                with warnings.catch_warnings(record=True):
                    # The deprecated commands should be ignored with
                    # a warning when they are parsed. Make sure that
                    # they will not cause any errors.
                    with self.assertWarns(KickstartDeprecationWarning) as cm:
                        parser.readKickstartFromString(command_name)

                    # Check the warning message.
                    expected = " {} command has been deprecated ".format(command_name)
                    self.assertIn(expected, str(cm.warning))

class RemovedCommandsParsing_TestCase(unittest.TestCase):
    def runTest(self):
        for version, command_map in control.commandMap.items():

            handler = makeVersion(version)
            parser = KickstartParser(handler)

            for command_name, command_class in command_map.items():
                if not issubclass(command_class, RemovedCommand):
                    continue

                # Make sure that using the removed command raises an error
                with self.assertRaises(KickstartParseError):
                    parser.readKickstartFromString(command_name)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2006, 2007, 2008, 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
"""
Base classes for creating commands and syntax version object.

This module exports several important base classes:

    BaseData - The base abstract class for all data objects.  Data objects
               are contained within a BaseHandler object.

    BaseHandler - The base abstract class from which versioned kickstart
                  handler are derived.  Subclasses of BaseHandler hold
                  BaseData and KickstartCommand objects.

    DeprecatedCommand - An abstract subclass of KickstartCommand that should
                        be further subclassed by users of this module.  When
                        a subclass is used, a warning message will be
                        printed.

    RemovedCommand - an abstract subclass of KickstartCommand that should
                     be subclassed to update the description with the version
                     it was removed in.
                     Any use of the command will raise an error.

    KickstartCommand - The base abstract class for all kickstart commands.
                       Command objects are contained within a BaseHandler
                       object.
"""
from pykickstart.i18n import _

import six
import warnings
from pykickstart import __version__
from pykickstart.errors import KickstartParseError, KickstartParseWarning, KickstartDeprecationWarning
from pykickstart.ko import KickstartObject
from pykickstart.version import versionToString
from pykickstart.parser import Packages

###
### COMMANDS
###
class KickstartCommand(KickstartObject):
    """The base class for all kickstart commands.  This is an abstract class."""
    removedKeywords = []
    removedAttrs = []

    def __init__(self, writePriority=0, *args, **kwargs):
        """Create a new KickstartCommand instance.  This method must be
           provided by all subclasses, but subclasses must call
           KickstartCommand.__init__ first.  Instance attributes:

           currentCmd    -- The name of the command in the input file that
                            caused this handler to be run.
           currentLine   -- The current unprocessed line from the input file
                            that caused this handler to be run.
           handler       -- A reference to the BaseHandler subclass this
                            command is contained withing.  This is needed to
                            allow referencing of Data objects.
           lineno        -- The current line number in the input file.
           seen          -- If this command was ever used in the kickstart file,
                            this attribute will be set to True.  This allows
                            for differentiating commands that were omitted
                            from those that default to unset.
           writePriority -- An integer specifying when this command should be
                            printed when iterating over all commands' __str__
                            methods.  The higher the number, the later this
                            command will be written.  All commands with the
                            same priority will be written alphabetically.
        """

        # We don't want people using this class by itself.
        if self.__class__ is KickstartCommand:
            raise TypeError("KickstartCommand is an abstract class.")

        KickstartObject.__init__(self, *args, **kwargs)

        self.writePriority = writePriority

        # These will be set by the dispatcher.
        self.currentCmd = ""
        self.currentLine = ""
        self.handler = None
        self.lineno = 0
        self.seen = False

        # If a subclass provides a removedKeywords list, warn if the user
        # continues to use some of the removed keywords
        for arg in (kw for kw in self.removedKeywords if kw in kwargs):
            warnings.warn("The '%s' keyword has been removed." % arg, KickstartParseWarning, stacklevel=2)

    def __call__(self, *args, **kwargs):
        """Set multiple attributes on a subclass of KickstartCommand at once
           via keyword arguments.  Valid attributes are anything specified in
           a subclass, but unknown attributes will be ignored.
        """
        self.seen = True

        for (key, val) in list(kwargs.items()):
            # Ignore setting attributes that were removed in a subclass, as
            # if they were unknown attributes.
            if key in self.removedAttrs:
                continue

            if hasattr(self, key):
                setattr(self, key, val)

    def __str__(self):
        """Return a string formatted for output to a kickstart file.  This
           method must be provided by all subclasses.
        """
        return KickstartObject.__str__(self)

    # pylint: disable=unused-argument
    def parse(self, args):
        """Parse the list of args and set data on the KickstartCommand object.
           This method must be provided by all subclasses.
        """
        raise TypeError("parse() not implemented for KickstartCommand")
    # pylint: enable=unused-argument

    def dataList(self):
        """For commands that can occur multiple times in a single kickstart
           file (like network, part, etc.), return the list that we should
           append more data objects to.
        """
        return None

    @property
    def dataClass(self):
        """For commands that can occur multiple times in a single kickstart
           file, return the class that should be used to store the data from
           each invocation.  An instance of this class will be appended to
           dataList.  For all other commands, return None.
        """
        return None

    def deleteRemovedAttrs(self):
        """Remove all attributes from self that are given in the removedAttrs
           list.  This method should be called from __init__ in a subclass,
           but only after the superclass's __init__ method has been called.
        """
        for attr in [k for k in self.removedAttrs if hasattr(self, k)]:
            delattr(self, attr)

    def set_to_self(self, namespace):
        """Set the contents of the namespace object (an instance of argparse.Namespace
           returned by parse_arguments) as attributes on the KickstartCommand object.
           It's useful to call this from KickstartCommand subclasses after parsing
           the arguments.
        """
        self.set_to_obj(namespace, self)

    # Just calls set_to_self - exists for backwards compatibility.
    def _setToSelf(self, namespace):
        warnings.warn("_setToSelf has been renamed to set_to_self.  The old name will be removed in a future release.", PendingDeprecationWarning, stacklevel=2)
        self.set_to_self(namespace)

    def set_to_obj(self, namespace, obj):
        """Sets the contents of the namespace object (an instance of argparse.Namespace
           returned by parse_arguments) as attributes on the provided object obj.  It's
           useful to call this from KickstartCommand subclasses that handle lists
           of objects (like partitions, network devices, etc.) and need to populate
           a Data object.
        """
        for (key, val) in vars(namespace).items():
            if val is not None:
                setattr(obj, key, val)

    # Just calls set_to_obj - exists for backwards compatibility.
    def _setToObj(self, namespace, obj):
        warnings.warn("_setToObj has been renamed to set_to_obj.  The old name will be removed in a future release.", PendingDeprecationWarning, stacklevel=2)
        self.set_to_obj(namespace, obj)

class DeprecatedCommand(KickstartCommand):
    """Specify that a command is deprecated and no longer has any function.
       Any command that is deprecated should be subclassed from this class,
       only specifying an __init__ method that calls the superclass's __init__.
       This is an abstract class.
    """
    def __init__(self, writePriority=None, *args, **kwargs):
        # We don't want people using this class by itself.
        if self.__class__ is DeprecatedCommand:
            raise TypeError("DeprecatedCommand is an abstract class.")

        # Create a new DeprecatedCommand instance.
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

    def dataList(self):
        """Override the method of the deprecated command."""
        return None

    @property
    def dataClass(self):
        """Override the attribute of the deprecated command."""
        return None

    def __str__(self):
        """Placeholder since DeprecatedCommands don't work anymore."""
        return ""

    def parse(self, args):
        """Print a warning message if the command is seen in the input file."""
        mapping = {"lineno": self.lineno, "cmd": self.currentCmd}
        warnings.warn(_("Ignoring deprecated command on line %(lineno)s:  The %(cmd)s command has been deprecated and no longer has any effect.  It may be removed from future releases, which will result in a fatal error from kickstart.  Please modify your kickstart file to remove this command.") % mapping, KickstartDeprecationWarning)

class RemovedCommand(KickstartCommand):
    """Specify that a command has been removed and no longer has any function.
       Any command that is removed should be subclassed from this class, and
       should set its description to add the version that the command was removed in.
       This is an abstract class.
    """
    def __init__(self, writePriority=None, *args, **kwargs):
        # We don't want people using this class by itself.
        if self.__class__ is RemovedCommand:
            raise TypeError("RemovedCommand is an abstract class.")

        # Create a new RemovedCommand instance.
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

    def dataList(self):
        """Override the method of the removed command."""
        return None

    @property
    def dataClass(self):
        """Override the attribute of the removed command."""
        return None

    def __str__(self):
        """Placeholder since RemovedCommands don't work anymore."""
        return ""

    def parse(self, args):
        """Raise an error if the command is found in the input file"""
        raise KickstartParseError(_("%s has been removed.") % self.currentCmd, lineno=self.lineno)

###
### HANDLERS
###
class KickstartHandler(KickstartObject):
    """An empty kickstart handler.

       This handler doesn't handle anything by default.

       version -- The version this syntax handler supports.  This is set by
                  a class attribute of a KickstartHandler subclass and is used to
                  set up the command dict.  It is for read-only use.
    """
    version = None

    def __init__(self, *args, **kwargs):
        """Create a new KickstartHandler instance.

           Instance attributes:

           commands -- A mapping from a string command to a KickstartCommand
                       subclass object that handles it.  Multiple strings can
                       map to the same object, but only one instance of the
                       command object should ever exist.  Most users should
                       never have to deal with this directly, as it is
                       manipulated internally and called through dispatcher.
           currentLine -- The current unprocessed line from the input file
                          that caused this handler to be run.
        """
        KickstartObject.__init__(self, *args, **kwargs)

        # These will be set by the dispatcher.
        self.commands = {}
        self.currentLine = ""

        # A dict keyed by an integer priority number, with each value being a
        # list of KickstartCommand subclasses.  This dict is maintained by
        # registerCommand and used in __str__.  No one else should be touching
        # it.
        self._writeOrder = {}

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        retval = ""

        lst = list(self._writeOrder.keys())
        lst.sort()

        for prio in lst:
            for obj in self._writeOrder[prio]:
                obj_str = obj.__str__()
                if isinstance(obj_str, six.text_type) and not six.PY3:
                    obj_str = obj_str.encode("utf-8")
                retval += obj_str

        return retval

    def _insertSorted(self, lst, obj):
        length = len(lst)
        i = 0

        while i < length:
            # If the two classes have the same name, it's because we are
            # overriding an existing class with one from a later kickstart
            # version, so remove the old one in favor of the new one.
            if obj.__class__.__name__ > lst[i].__class__.__name__:
                i += 1
            elif obj.__class__.__name__ == lst[i].__class__.__name__:
                lst[i] = obj
                return
            elif obj.__class__.__name__ < lst[i].__class__.__name__:
                break

        if i >= length:
            lst.append(obj)
        else:
            lst.insert(i, obj)

    def _setCommand(self, cmdObj):
        # Add an attribute on this version object.  We need this to provide a
        # way for clients to access the command objects.  We also need to strip
        # off the version part from the front of the name.
        if cmdObj.__class__.__name__.find("_") != -1:
            name = cmdObj.__class__.__name__.split("_", 1)[1]
            if not six.PY3:
                name = unicode(name)    # pylint: disable=undefined-variable
        else:
            name = cmdObj.__class__.__name__.lower()
            if not six.PY3:
                name = unicode(name)    # pylint: disable=undefined-variable

        setattr(self, name.lower(), cmdObj)

        # Also, add the object into the _writeOrder dict in the right place.
        if cmdObj.writePriority is not None:
            if cmdObj.writePriority in self._writeOrder:
                self._insertSorted(self._writeOrder[cmdObj.writePriority], cmdObj)
            else:
                self._writeOrder[cmdObj.writePriority] = [cmdObj]

    def registerCommand(self, cmdName, cmdClass):
        # First make sure we haven't instantiated this command handler
        # already.  If we have, we just need to make another mapping to
        # it in self.commands.
        # NOTE:  We can't use the resetCommand method here since that relies
        # upon cmdClass already being instantiated.  We'll just have to keep
        # these two code blocks in sync.
        cmdObj = None

        for (_key, val) in list(self.commands.items()):
            if val.__class__.__name__ == cmdClass.__name__:
                cmdObj = val
                break

        # If we didn't find an instance in self.commands, create one now.
        if cmdObj is None:
            cmdObj = cmdClass()
            self._setCommand(cmdObj)

        # Finally, add the mapping to the commands dict.
        self.commands[cmdName] = cmdObj
        self.commands[cmdName].handler = self

    def registerData(self, dataName, dataClass):
        # We also need to create attributes for the various data objects.
        setattr(self, dataName, dataClass)

    def resetCommand(self, cmdName):
        """Given the name of a command that's already been instantiated, create
           a new instance of it that will take the place of the existing
           instance.  This is equivalent to quickly blanking out all the
           attributes that were previously set.

           This method raises a KeyError if cmdName is invalid.
        """
        if cmdName not in self.commands:
            raise KeyError

        cmdObj = self.commands[cmdName].__class__()

        self._setCommand(cmdObj)
        self.commands[cmdName] = cmdObj
        self.commands[cmdName].handler = self

    def dispatcher(self, args, lineno):
        """Call the appropriate KickstartCommand handler for the current line
           in the kickstart file.  A handler for the current command should
           be registered, though a handler of None is not an error.  Returns
           the data object returned by KickstartCommand.parse.

           args    -- A list of arguments to the current command
           lineno  -- The line number in the file, for error reporting
        """
        cmd = args[0]

        if cmd not in self.commands:
            raise KickstartParseError(_("Unknown command: %s") % cmd, lineno=lineno)
        elif self.commands[cmd] is not None:
            self.commands[cmd].currentCmd = cmd
            self.commands[cmd].currentLine = self.currentLine
            self.commands[cmd].lineno = lineno
            self.commands[cmd].seen = True

            # The parser returns the data object that was modified.  This is either
            # the command handler object itself (a KickstartCommand object), or it's
            # a BaseData subclass instance that should be put into the command's
            # dataList.  The latter is done via side effects.
            #
            # Regardless, return the object that was given to us by the parser.
            obj = self.commands[cmd].parse(args[1:])

            # Here's the side effect part - don't worry about lst not being returned.
            lst = self.commands[cmd].dataList()
            if isinstance(obj, BaseData) and lst is not None:
                lst.append(obj)

            return obj


class BaseHandler(KickstartHandler):
    """A base kickstart handler.

       Each version of kickstart syntax is provided by a subclass of this
       class. These subclasses are what users will interact with for parsing,
       extracting data, and writing out kickstart files.  This is an abstract
       class.
    """

    def __init__(self, mapping=None, dataMapping=None, commandUpdates=None,
                 dataUpdates=None, *args, **kwargs):
        """Create a new BaseHandler instance.  This method must be provided by
           all subclasses, but subclasses must call BaseHandler.__init__ first.

           mapping          -- A custom map from command strings to classes,
                               useful when creating your own handler with
                               special command objects.  It is otherwise unused
                               and rarely needed.  If you give this argument,
                               the mapping takes the place of the default one
                               and so must include all commands you want
                               recognized.
           dataMapping      -- This is the same as mapping, but for data
                               objects.  All the same comments apply.
           commandUpdates   -- This is similar to mapping, but does not take
                               the place of the defaults entirely.  Instead,
                               this mapping is applied after the defaults and
                               updates it with just the commands you want to
                               modify.
           dataUpdates      -- This is the same as commandUpdates, but for
                               data objects.


           Instance attributes:

           packages -- An instance of pykickstart.parser.Packages which
                       describes the packages section of the input file.
           platform -- A string describing the hardware platform, which is
                       needed only by system-config-kickstart.
           scripts  -- A list of pykickstart.parser.Script instances, which is
                       populated by KickstartParser.addScript and describes the
                       %pre/%pre-install/%post/%traceback script section of the
                       input file.
        """

        # We don't want people using this class by itself.
        if self.__class__ is BaseHandler:
            raise TypeError("BaseHandler is an abstract class.")

        KickstartHandler.__init__(self, *args, **kwargs)

        # This isn't really a good place for these, but it's better than
        # everything else I can think of.
        self.scripts = []
        self.packages = Packages()
        self.platform = ""

        # Any sections that we do not understand but want to prevent causing errors
        # are represented by a NullSection.  We want to preserve those on output, so
        # keep a list of their string representations here.  This is likely to change
        # in the future.  Don't rely on this exact implementation.
        self._null_section_strings = []

        self._registerCommands(mapping, dataMapping, commandUpdates, dataUpdates)

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        retval = "# Generated by pykickstart v%s\n" % __version__

        if self.platform:
            retval += "#platform=%s\n" % self.platform

        retval += "#version=%s\n" % versionToString(self.version)

        retval += KickstartHandler.__str__(self)

        for script in self.scripts:
            script_str = script.__str__()
            if isinstance(script_str, six.text_type) and not six.PY3:
                script_str = script_str.encode("utf-8")
            retval += script_str

        if self._null_section_strings:
            retval += "\n"

            for s in self._null_section_strings:
                retval += s

        retval += self.packages.__str__()

        return retval

    def _registerCommands(self, mapping=None, dataMapping=None, commandUpdates=None,
                          dataUpdates=None):
        if mapping == {} or mapping is None:
            from pykickstart.handlers.control import commandMap
            cMap = commandMap[self.version]
        else:
            cMap = mapping

        if dataMapping == {} or dataMapping is None:
            from pykickstart.handlers.control import dataMap
            dMap = dataMap[self.version]
        else:
            dMap = dataMapping

        # Apply the command and data updates, but do
        # not modify the original command and data maps.
        if isinstance(commandUpdates, dict):
            cMap = dict(cMap)
            cMap.update(commandUpdates)

        if isinstance(dataUpdates, dict):
            dMap = dict(dMap)
            dMap.update(dataUpdates)

        for (cmdName, cmdClass) in list(cMap.items()):
            self.registerCommand(cmdName, cmdClass)

        # No checks here because dMap is a bijection.  At least, that's what
        # the comment says.  Hope no one screws that up.
        for (dataName, dataClass) in list(dMap.items()):
            self.registerData(dataName, dataClass)

    def maskAllExcept(self, lst):
        """Set all entries in the commands dict to None, except the ones in
           the lst.  All other commands will not be processed.
        """
        self._writeOrder = {}

        for (key, _val) in list(self.commands.items()):
            if key not in lst:
                self.commands[key] = None

    def hasCommand(self, cmd):
        """Return true if there is a handler for the string cmd."""
        return hasattr(self, cmd)

###
### DATA
###
class BaseData(KickstartObject):
    """The base class for all data objects.  This is an abstract class."""
    removedKeywords = []
    removedAttrs = []

    def __init__(self, *args, **kwargs):
        """Create a new BaseData instance.

           lineno -- Line number in the ks-file where this object was defined
        """

        # We don't want people using this class by itself.
        if self.__class__ is BaseData:
            raise TypeError("BaseData is an abstract class.")

        KickstartObject.__init__(self, *args, **kwargs)
        self.lineno = 0

        # If a subclass provides a removedKeywords list, warn if the user
        # continues to use some of the removed keywords
        for arg in (kw for kw in self.removedKeywords if kw in kwargs):
            warnings.warn("The '%s' keyword has been removed." % arg, KickstartParseWarning, stacklevel=2)

    def __str__(self):
        """Return a string formatted for output to a kickstart file."""
        return ""

    def __call__(self, *args, **kwargs):
        """Set multiple attributes on a subclass of BaseData at once via
           keyword arguments.  Valid attributes are anything specified in a
           subclass, but unknown attributes will be ignored.
        """
        for (key, val) in list(kwargs.items()):
            # Ignore setting attributes that were removed in a subclass, as
            # if they were unknown attributes.
            if key in self.removedAttrs:
                continue

            if hasattr(self, key):
                setattr(self, key, val)

    def deleteRemovedAttrs(self):
        """Remove all attributes from self that are given in the removedAttrs
           list.  This method should be called from __init__ in a subclass,
           but only after the superclass's __init__ method has been called.
        """
        for attr in [k for k in self.removedAttrs if hasattr(self, k)]:
            delattr(self, attr)



import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.unsupported_hardware import RHEL6_UnsupportedHardware

class RHEL6_TestCase(CommandTest):
    command = "unsupported_hardware"

    def runTest(self):
        # pass
        self.assert_parse("unsupported_hardware",
                          "unsupported_hardware\n")

        # fail
        self.assert_parse_error("unsupported_hardware --cheese")
        self.assert_parse_error("unsupported_hardware cheese")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.unsupported_hardware = False
        self.assertEqual(cmd.__str__(), "")

class Unsupported_Hardware_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = RHEL6_UnsupportedHardware()
        self.assertEqual(cmd.unsupported_hardware, False)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC6
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser, commaSplit

from pykickstart.i18n import _

class FC6_Services(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.disabled = kwargs.get("disabled", [])
        self.enabled = kwargs.get("enabled", [])

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        args = ""

        if self.disabled:
            args += " --disabled=\"%s\"" % ",".join(self.disabled)
        if self.enabled:
            args += " --enabled=\"%s\"" % ",".join(self.enabled)

        if args:
            retval += "# System services\nservices%s\n" % args

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="services", description="""
                            Modifies the default set of services that will run
                            under the default runlevel. The services listed in
                            the disabled list will be disabled before the
                            services listed in the enabled list are enabled.
                            """, epilog="""
                            One of ``--disabled`` or ``--enabled`` must be provided.
                            """, version=FC6)
        op.add_argument("--disabled", type=commaSplit, version=FC6,
                        metavar="<list>", help="""
                        Disable the services given in the comma separated list.
                        """)
        op.add_argument("--enabled", type=commaSplit, version=FC6,
                        metavar="<list>", help="""
                        Enable the services given in the comma separated list.
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if not (self.disabled or self.enabled):
            raise KickstartParseError(_("One of --disabled or --enabled must be provided."), lineno=self.lineno)

        return self



from distutils.command.install_scripts import install_scripts as _install_scripts
from distutils.file_util import move_file
import os
from setuptools import setup


# This is required so that the scripts are detected by code coverage during testing, but are
# installed without their filename extensions
class install_scripts(_install_scripts):
    def run(self):
        _install_scripts.run(self)

        for old in self.get_outputs():
            new = os.path.splitext(old)[0]
            # move_file doesn't have a way to tell it to overwrite, so remove
            # the existing copy to accomplish the same thing.
            if os.path.isfile(new) and os.path.exists(new):
                os.unlink(new)
            move_file(old, new)


setup(cmdclass={"install_scripts": install_scripts},
      name='pykickstart', version='3.32.3',
      description='Python module for manipulating kickstart files',
      author='Chris Lumens', author_email='clumens@redhat.com',
      url='http://fedoraproject.org/wiki/pykickstart',
      install_requires=['six', 'requests'],
      extras_require={
          "docs": ['Sphinx'],
          "test": ['coveralls', 'coverage', 'pocketlint', 'pylint']},
      # These are installed by install_scripts() without their filename extensions
      scripts=['tools/ksvalidator.py', 'tools/ksflatten.py', 'tools/ksverdiff.py', 'tools/ksshell.py'],
      packages=['pykickstart', 'pykickstart.commands', 'pykickstart.handlers'],
      data_files=[('share/man/man1', ['docs/ksvalidator.1', 'docs/ksflatten.1', 'docs/ksverdiff.1',
                                      'docs/ksshell.1'])],
      classifiers=["Programming Language :: Python :: 3"])



#
# Copyright 2019 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.constants import SECURE_BOOT_AUTO, SECURE_BOOT_ENABLED, SECURE_BOOT_DISABLED, \
    SECURE_BOOT_DEFAULT
from pykickstart.version import F32
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser


class F32_Zipl(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.secure_boot = kwargs.get("secure_boot", SECURE_BOOT_DEFAULT)

    def _getParser(self):
        op = KSOptionParser(prog="zipl", version=F32, description="""
                            This command specifies the ZIPL configuration for s390x.
                            """)
        op.add_argument("--secure-boot", dest="secure_boot", version=F32,
                        action="store_const", const=SECURE_BOOT_AUTO,
                        help="""
                        Enable Secure Boot if supported by the installing machine.

                        **Note** When installed on a model newer than IBM z14, the
                        installed system cannot be booted from an IBM z14 and earlier
                        models.
                        """)
        op.add_argument("--force-secure-boot", dest="secure_boot", version=F32,
                        action="store_const", const=SECURE_BOOT_ENABLED,
                        help="""
                        Enable Secure Boot unconditionally.

                        **Note** Installation will fail on IBM z14 and earlier models.
                        """)
        op.add_argument("--no-secure-boot", dest="secure_boot", version=F32,
                        action="store_const", const=SECURE_BOOT_DISABLED,
                        help="""
                        Disable Secure Boot.

                        **Note** Secure Boot is not supported on IBM z14 and earlier
                        models, therefore choose '--no-secure-boot' if you intend to
                        boot the installed system on such models.
                        """)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.secure_boot is not SECURE_BOOT_DEFAULT:
            retval += "# ZIPL configuration\nzipl"
            retval += self._getArgsAsStr() + "\n"

        return retval

    def _getArgsAsStr(self):
        ret = ""

        if self.secure_boot == SECURE_BOOT_AUTO:
            ret += " --secure-boot"
        elif self.secure_boot == SECURE_BOOT_ENABLED:
            ret += " --force-secure-boot"
        elif self.secure_boot == SECURE_BOOT_DISABLED:
            ret += " --no-secure-boot"

        return ret


class RHEL8_Zipl(F32_Zipl):
    pass



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F8, RHEL6, F29, F34
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.i18n import _
from pykickstart.options import KSOptionParser, commaSplit

class FC3_IgnoreDisk(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.ignoredisk = kwargs.get("ignoredisk", [])

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.ignoredisk:
            retval += "ignoredisk --drives=%s\n" % ",".join(self.ignoredisk)

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="ignoredisk", description="""
            Controls anaconda's access to disks attached to the system. By
            default, all disks will be available for partitioning. Only one of
            the following three options may be used.""", version=FC3)
        op.add_argument("--drives", dest="ignoredisk", type=commaSplit,
                        required=True, version=FC3, help="""
                        Specifies those disks that anaconda should not touch
                        when partitioning, formatting, and clearing.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F8_IgnoreDisk(FC3_IgnoreDisk):
    removedKeywords = FC3_IgnoreDisk.removedKeywords
    removedAttrs = FC3_IgnoreDisk.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_IgnoreDisk.__init__(self, writePriority, *args, **kwargs)

        self.onlyuse = kwargs.get("onlyuse", [])

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.ignoredisk:
            retval += "ignoredisk --drives=%s\n" % ",".join(self.ignoredisk)
        elif self.onlyuse:
            retval += "ignoredisk --only-use=%s\n" % ",".join(self.onlyuse)

        return retval

    def parse(self, args):
        retval = FC3_IgnoreDisk.parse(self, args)

        howmany = 0
        if self.ignoredisk:
            howmany += 1
        if self.onlyuse:
            howmany += 1
        if howmany != 1:
            raise KickstartParseError(_("One of --drives or --only-use must be specified "
                                        "for ignoredisk command."), lineno=self.lineno)

        return retval

    def _getParser(self):
        op = FC3_IgnoreDisk._getParser(self)
        op.add_argument("--drives", dest="ignoredisk", version=F8,
                        type=commaSplit, help="""
                        This argument is no longer required!""")
        op.add_argument("--only-use", dest="onlyuse",
                        type=commaSplit, version=F8, help="""
                        Specifies the opposite - only disks listed here will be
                        used during installation.""")
        return op

class RHEL6_IgnoreDisk(F8_IgnoreDisk):
    removedKeywords = F8_IgnoreDisk.removedKeywords
    removedAttrs = F8_IgnoreDisk.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        F8_IgnoreDisk.__init__(self, writePriority, *args, **kwargs)
        self.interactive = kwargs.get("interactive", False)

    def __str__(self):
        retval = F8_IgnoreDisk.__str__(self)

        if self.interactive:
            retval = "ignoredisk --interactive\n"

        return retval

    def parse(self, args):
        retval = FC3_IgnoreDisk.parse(self, args)

        howmany = 0
        if self.ignoredisk:
            howmany += 1
        if self.onlyuse:
            howmany += 1
        if self.interactive:
            howmany += 1
        if howmany != 1:
            raise KickstartParseError(_("One of --drives , --only-use , or --interactive must be specified for ignoredisk command."), lineno=self.lineno)

        if self.interactive:
            self.ignoredisk = []

        return retval

    def _getParser(self):
        op = F8_IgnoreDisk._getParser(self)
        op.add_argument("--interactive", action="store_true",
                        default=False, version=RHEL6, help="""
                        Allow the user manually navigate the advanced storage
                        screen.""")
        return op

class F14_IgnoreDisk(RHEL6_IgnoreDisk):
    pass

class F29_IgnoreDisk(F14_IgnoreDisk):
    removedKeywords = F14_IgnoreDisk.removedKeywords
    removedAttrs = F14_IgnoreDisk.removedAttrs

    def parse(self, args):
        return F8_IgnoreDisk.parse(self, args)

    def _getParser(self):
        op = F14_IgnoreDisk._getParser(self)
        op.add_argument("--interactive", action="store_true",
                        default=False, deprecated=F29, help="""
                        Allow the user manually navigate the advanced storage
                        screen.""")
        return op

class F34_IgnoreDisk(F29_IgnoreDisk):
    removedKeywords = F29_IgnoreDisk.removedKeywords
    removedAttrs = F29_IgnoreDisk.removedAttrs

    def _getParser(self):
        op = F29_IgnoreDisk._getParser(self)
        op.remove_argument("--interactive", version=F34)
        return op



#
# Brian C. Lane <bcl@redhat.com>
#
# Copyright 2012 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import RHEL6
from pykickstart.base import KickstartCommand
from pykickstart.options import KSOptionParser

class RHEL6_UnsupportedHardware(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()
        self.unsupported_hardware = kwargs.get("unsupported_hardware", False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.unsupported_hardware:
            retval += "unsupported_hardware\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="unsupported_hardware", description="allow installation to proceed on systems with tainted hardware", version=RHEL6)
        return op

    def parse(self, args):
        self.op.parse_args(args=args, lineno=self.lineno)
        self.unsupported_hardware = True
        return self



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
import warnings
from tests.baseclass import CommandTest
from pykickstart.base import DeprecatedCommand, RemovedCommand
from pykickstart.commands.upgrade import FC3_Upgrade

class Upgrade_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_Upgrade()
        self.assertEqual(cmd.__str__(), '')

class FC3_TestCase(CommandTest):
    command = "upgrade"

    def runTest(self):
        # pass
        self.assert_parse("upgrade", "upgrade\n")
        self.assert_parse("install", "install\n")

        # fail
        self.assert_parse_error("upgrade install")
        self.assert_parse_error("upgrade --bad-flag")
        self.assert_parse_error("install --bad-flag")


class F11_TestCase(FC3_TestCase):
    def runTest(self):
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("upgrade", "upgrade\n")
        self.assert_parse("install", "install\n")
        self.assert_parse("upgrade --root-device=/dev/sda", "upgrade --root-device=/dev/sda\n")
        self.assert_parse("install --root-device=/dev/sda", "install\n")

        # fail
        # --root-device requires argument
        self.assert_parse_error("upgrade --root-device")
        self.assert_parse_error("upgrade --root-device=\"\"")
        # unknown option
        self.assert_parse_error("upgrade --bad-flag")

class F20_TestCase(F11_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("upgrade")
        self.assertEqual(issubclass(parser.__class__, DeprecatedCommand), True)

        # deprecated commands don't return a useful string - test that somewhere
        self.assertEqual(str(self.getParser("upgrade")), "")

        # deprecated commands also raise a deprecation warning - test that somewhere
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")
            # The upgrade command takes no arguments and we've already got the object
            # that knows how to parse it from the beginning of this function.
            parser.parse([])
            self.assertEqual(len(w), 1)

class F29_TestCase(F20_TestCase):
    def runTest(self):
        # make sure that upgrade is removed
        cmd = self.handler().commands["upgrade"]
        self.assertTrue(issubclass(cmd.__class__, RemovedCommand))

class RHEL8_TestCase(F20_TestCase):
    def runTest(self):
        # make sure that upgrade is removed
        self.assertNotIn("upgrade", self.handler().commands)

if __name__ == "__main__":
    unittest.main()



import unittest
import importlib
from textwrap import dedent

from pykickstart.version import *           # pylint: disable=wildcard-import
from pykickstart.handlers import control
from pykickstart.base import KickstartCommand, BaseData, BaseHandler


class HandlerMappingTestCase(unittest.TestCase):
    """
        Verify that command handlers actually map to the
        latest possible commands implementations. See:
        https://github.com/rhinstaller/pykickstart/issues/28
    """
    def runTest(self):
        errors = 0
        for handler_version in control.commandMap:
            # test for commands
            for command_name, command_class in control.commandMap[handler_version].items():
                command_version = getVersionFromCommandClass(command_class)

                # skip commands with the same version as their handler
                # b/c they are obviously the latest possible version
                if command_version == handler_version:
                    continue

                # load the module defining all possible command implementations
                command_module = importlib.import_module(command_class.__module__)

                for impl_class in command_module.__dict__.values():
                    # skip everything which isn't a class
                    if type(impl_class) is not type:
                        continue

                    # skip everything which doesn't inherit from KickstartCommand
                    if not issubclass(impl_class, KickstartCommand):
                        continue

                    # skip base classes as well
                    if impl_class.__name__ in ['KickstartCommand', 'DeprecatedCommand', 'RemovedCommand']:
                        continue

                    # version of this particular implementation
                    impl_version = getVersionFromCommandClass(impl_class)

                    # skip older implementations of the same command
                    if impl_version <= command_version:
                        continue

                    # if there is a newer implementation, which isn't newer than the
                    # handler version this means our commandMap doesn't contain the
                    # latest possible version for this command. For example we're testing
                    # the f23.py handler so handler_version == F23, command_version == F20
                    # and impl_version == F21 or later. This is a bug!
                    if command_version < impl_version and impl_version <= handler_version:
                        s_handler_version = versionToString(handler_version, True)
                        s_impl_version = versionToString(impl_version, True)
                        # skip newer RHEL implementations if testing a Fedora handler
                        # or vice versa
                        if (s_handler_version.startswith('F') and \
                            s_impl_version.startswith('RHEL')) or \
                            (s_handler_version.startswith('RHEL') and \
                            s_impl_version.startswith('F')):
                            continue

                        errors += 1
                        message = dedent("""
                            ERROR: In `handlers/%s.py` the "%s" command maps to "%s" while in
                            `%s` there is newer implementation: "%s".
                        """ % (versionToString(handler_version, True).lower(),
                               command_name, command_class.__name__,
                               command_class.__module__, impl_class.__name__))
                        print(message)

            # test for data
            for data_name, data_class in control.dataMap[handler_version].items():
                data_version = getVersionFromCommandClass(data_class)

                # skip data with the same version as their handler
                # b/c they are obviously the latest possible version
                if data_version == handler_version:
                    continue

                # load the module defining all possible data implementations
                data_module = importlib.import_module(data_class.__module__)

                for impl_class in data_module.__dict__.values():
                    # skip everything which isn't a class
                    if type(impl_class) is not type:
                        continue

                    # skip everything which doesn't inherit from BaseData
                    if not issubclass(impl_class, BaseData):
                        continue

                    # skip base classes as well
                    if impl_class.__name__ == 'BaseData':
                        continue

                    # version of this particular implementation
                    impl_version = getVersionFromCommandClass(impl_class)

                    # skip older implementations of the same data
                    if impl_version <= data_version:
                        continue

                    # if there is a newer implementation, which isn't newer than the
                    # handler version this means our dataMap doesn't contain the
                    # latest possible version for this data.
                    if data_version < impl_version and impl_version <= handler_version:
                        s_handler_version = versionToString(handler_version, True)
                        s_impl_version = versionToString(impl_version, True)
                        # skip newer RHEL implementations if testing a Fedora handler
                        # or vice versa
                        if (s_handler_version.startswith('F') and \
                            s_impl_version.startswith('RHEL')) or \
                            (s_handler_version.startswith('RHEL') and \
                            s_impl_version.startswith('F')):
                            continue

                        errors += 1
                        message = dedent("""
                            ERROR: In `handlers/%s.py` "%s" maps to "%s" while in
                            `%s` there is newer implementation: "%s".
                        """ % (versionToString(handler_version, True).lower(),
                               data_name, data_class.__name__,
                               data_class.__module__, impl_class.__name__))
                        print(message)

        # assert for errors presence
        self.assertEqual(0, errors)


class HandlerUpdateTestCase(unittest.TestCase):

    class TestHandler(BaseHandler):
        version = F27

    class TestCommand1(KickstartCommand):
        pass

    class TestCommand2(KickstartCommand):
        pass

    class TestData1(BaseData):
        pass

    class TestData2(BaseData):
        pass

    def setUp(self):
        version_cls = self._get_version_cls()
        self.command_map = dict(version_cls.commandMap)
        self.data_map = dict(version_cls.dataMap)

    def assertEmpty(self, tested_set):
        self.assertEqual(len(tested_set), 0, msg="The set is not empty.")

    def _get_version_cls(self):
        version = self.TestHandler.version
        return returnClassForVersion(version)

    def _test_handler(self, command_updates, data_updates):
        # Create a new handler
        handler = self.TestHandler(commandUpdates=command_updates,
                                   dataUpdates=data_updates)

        # Check that the specified version has not changed its maps.
        version_cls = self._get_version_cls()
        self.assertEqual(self.command_map, version_cls.commandMap)
        self.assertEqual(self.data_map, version_cls.dataMap)

        # Check that the handler knows all commands from the specified version.
        self.assertEmpty(set(self.command_map)
                         - set(handler.commands))

        # Check that the handler knows all commands from updates.
        self.assertEmpty(set(command_updates)
                         - set(handler.commands))

        # Check that the handler doesn't know anything else.
        self.assertEmpty(set(handler.commands)
                         - set(self.command_map)
                         - set(handler.commands))

        # Check parents of the commands in the handler.
        command_map = dict(self.command_map)
        command_map.update(command_updates)

        for name, obj in handler.commands.items():
            self.assertTrue(isinstance(obj, command_map[name]))

        # Check the data.
        data_map = dict(self.data_map)
        data_map.update(data_updates)

        for name, cls in data_map.items():
            self.assertEqual(getattr(handler, name), cls)

    def test_no_updates(self):
        """Test handler with no updates."""
        self._test_handler(dict(), dict())

    def test_command_updates(self):
        """Test handler with command updates."""
        command_updates = {
            "autopart": self.TestCommand1,
            "nonexistent": self.TestCommand2
        }

        self._test_handler(command_updates, dict())
        self._test_handler(dict(), dict())

    def test_data_updates(self):
        """Test handler with data updates."""
        data_updates = {
            "UserData": self.TestData1,
            "NonExistent": self.TestData2
        }

        self._test_handler(dict(), data_updates)
        self._test_handler(dict(), dict())

    def test_all_updates(self):
        """Test handler with command and data updates."""
        command_updates = {
            "bootloader": self.TestCommand1,
            "user": self.TestCommand2,
            "imaginary": self.TestCommand1
        }

        data_updates = {
            "RepoData": self.TestData1,
            "Imaginary": self.TestData2,
            "FakeData": self.TestData2
        }

        self._test_handler(command_updates, data_updates)
        self._test_handler(dict(), dict())

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F34, versionToLongString
from pykickstart.base import KickstartCommand, DeprecatedCommand
from pykickstart.options import KSOptionParser

class FC3_Method(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    # These are all set up as part of the base KickstartCommand.  We want to
    # make sure looking them up gets redirected to the right place.
    internals = ["method",
                 "writePriority", "currentCmd", "currentLine", "handler", "lineno", "seen"]

    _methods = ["cdrom", "harddrive", "nfs", "url"]

    def _clear_seen(self):
        """ Reset all the method's seen attributes to False"""
        for method in self._methods:
            setattr(getattr(self.handler, method), "seen", False)

    def _get_command(self, method):
        """Get a command for the given method."""
        if method is None:
            # We use this to select the closest mirror option.
            # TODO: Raise an error instead.
            method = "url"

        if method not in self._methods:
            raise AttributeError("Unknown method: {}".format(method))

        return getattr(self.handler, method)

    @property
    def method(self):
        """Return the seen method or None."""
        for method in self._methods:
            if getattr(self.handler, method).seen:
                return method

        return None

    @method.setter
    def method(self, value):
        """Set which method is seen."""
        self._clear_seen()

        if value is None:
            return

        self._get_command(value).seen = True

    def __getattr__(self, name):
        """Get the attribute in the seen command.

        Called when an attribute lookup has not found
        the attribute in the usual places.
        """
        # Prevent recursion in copy and deepcopy.
        # See commit f5dbcfb for explanation.
        if name in ["handler", "method", "_methods"]:
            raise AttributeError()

        command = self._get_command(self.method)
        return getattr(command, name)

    def __setattr__(self, name, value):
        """Set the attribute in the seen command.

        Called when an attribute assignment is attempted.
        """
        if name in self.internals:
            super(FC3_Method, self).__setattr__(name, value)
            return

        command = self._get_command(self.method)
        # Check if the kickstart command has the attribute.
        # Instead of using hasattr, that calls getattr and catches the exception
        # that we would have to raise again, we call getattr directly.
        getattr(command, name)
        # Set the attribute of the kickstart command.
        command.__setattr__(name, value)

    def _getParser(self):
        """Return a parser."""

        description = "Proxy to the actual installation method. Valid installation methods are:\n\n"
        for method in self._methods:
            description += "* ``%s``\n" % method

        return KSOptionParser(prog="method", description=description, version=FC3)


# These are all just for compat.  Calling into the appropriate version-specific
# method command will deal with making sure the right options are used.
class FC6_Method(FC3_Method):
    pass

class F13_Method(FC6_Method):
    pass

class F14_Method(F13_Method):
    pass

class RHEL6_Method(F14_Method):
    pass

class F18_Method(F14_Method):
    pass

class F19_Method(FC3_Method):
    removedKeywords = FC3_Method.removedKeywords
    removedAttrs = FC3_Method.removedAttrs

    _methods = FC3_Method._methods + ["liveimg"]

class RHEL7_Method(F19_Method):
    removedKeywords = F19_Method.removedKeywords
    removedAttrs = F19_Method.removedAttrs

    _methods = F19_Method._methods + ["hmc"]

class F28_Method(RHEL7_Method):
    pass

class F34_Method(DeprecatedCommand, F19_Method):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = F19_Method._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F34)
        return op



#
# Copyright (C) 2014  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
from pykickstart.version import F21
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

class F21_OSTreeSetup(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, *args, **kwargs):
        KickstartCommand.__init__(self, *args, **kwargs)
        self.op = self._getParser()
        self.osname = kwargs.get('osname', None)
        self.remote = kwargs.get("remote", self.osname)
        self.url = kwargs.get('url', None)
        self.ref = kwargs.get('ref', None)
        self.nogpg = kwargs.get('nogpg', False)

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.osname:
            retval += "# OSTree setup\n"
            retval += "ostreesetup %s\n" % self._getArgsAsStr()

        return retval

    def _getArgsAsStr(self):
        retcmd = []
        if self.osname:
            retcmd.append('--osname="%s"' % self.osname)
        if self.remote:
            retcmd.append('--remote="%s"' % self.remote)
        if self.url:
            retcmd.append('--url="%s"' % self.url)
        if self.ref:
            retcmd.append('--ref="%s"' % self.ref)
        if self.nogpg:
            retcmd.append('--nogpg')
        return ' '.join(retcmd)

    def _getParser(self):
        op = KSOptionParser(prog="ostreesetup", description="""
                            Used for OSTree installations. See
                            https://wiki.gnome.org/action/show/Projects/OSTree
                            for more information about OSTree.
                            """, version=F21)
        op.add_argument("--osname", required=True, version=F21, help="""
                        Management root for OS installation.""")
        op.add_argument("--remote", version=F21, help="""
                        Management root for OS installation.""")
        op.add_argument("--url", required=True, version=F21, help="""
                        Repository URL.""")
        op.add_argument("--ref", required=True, version=F21, help="""
                        Name of branch inside the repository.""")
        op.add_argument("--nogpg", action="store_true", version=F21, help="""
                        Disable GPG key verification.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        if not self.remote:
            self.remote = self.osname

        if not self.url.startswith(("file:", "http:", "https:")):
            raise KickstartParseError("ostree repos must use file, HTTP or HTTPS protocol.", lineno=self.lineno)

        return self

class RHEL7_OSTreeSetup(F21_OSTreeSetup):
    pass

class RHEL8_OSTreeSetup(F21_OSTreeSetup):
    pass

class RHEL9_OSTreeSetup(F21_OSTreeSetup):
    pass



# Andy Lindeberg <alindebe@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.dmraid import FC6_DmRaidData
from pykickstart.base import DeprecatedCommand, RemovedCommand
from pykickstart.version import FC6

class DmRaid_TestCase(unittest.TestCase):
    def runTest(self):
        data1 = FC6_DmRaidData()
        data2 = FC6_DmRaidData()
        self.assertEqual(data1, data2)

        # test that objects which differ in one
        # of these attributes are not equal
        data1.name = ''
        data2.name = 'test'
        self.assertNotEqual(data1, data2)
        self.assertNotEqual(data2, data1)
        data1.name = ''
        data2.name = ''

        data1.devices = []
        data2.devices = ['test']
        self.assertNotEqual(data1, data2)
        self.assertNotEqual(data2, data1)
        data1.devices = []
        data2.devices = []


class FC6_TestCase(CommandTest):
    command = "dmraid"

    def runTest(self):
        # pass
        self.assert_parse("dmraid --name=/dev/onamai --dev=debaisi", "dmraid --name=onamai --dev=\"debaisi\"\n")
        self.assert_parse("dmraid --name onamai --dev debaisi", "dmraid --name=onamai --dev=\"debaisi\"\n")
        self.assert_parse("dmraid --dev=deb1,deb2 --name onamai", "dmraid --name=onamai --dev=\"deb1,deb2\"\n")
        self.assert_parse("dmraid --dev \"deb1,deb2\" --name=onamai", "dmraid --name=onamai --dev=\"deb1,deb2\"\n")

        # equality
        self.assertEqual(self.assert_parse("dmraid --name=raidA --dev=deviceA"), self.assert_parse("dmraid --name=raidA --dev=deviceA"))
        self.assertNotEqual(self.assert_parse("dmraid --name=raidA --dev=deviceA"), None)
        self.assertNotEqual(self.assert_parse("dmraid --name=raidA --dev=deviceA"), self.assert_parse("dmraid --name=raidB --dev=deviceA"))
        self.assertNotEqual(self.assert_parse("dmraid --name=raidA --dev=deviceA"), self.assert_parse("dmraid --name=raidA --dev=deviceB"))

        # fail
        self.assert_parse_error("dmraid")
        self.assert_parse_error("dmraid --name")
        self.assert_parse_error("dmraid --dev")
        self.assert_parse_error("dmraid --name=onamai")
        self.assert_parse_error("dmraid --name onamai")
        self.assert_parse_error("dmraid --dev debaisi")
        self.assert_parse_error("dmraid --dev=deb1,deb2")
        self.assert_parse_error("dmraid --magic")

        # extra test coverage
        cmd = self.handler().commands[self.command]
        cmd.dmraids = "--name=blah"
        self.assertEqual(cmd.__str__(), "--name=blah")

class FC6_Duplicate_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = FC6

    def runTest(self):
        self.assert_parse("""
dmraid --name=raidA --dev=deviceA
dmraid --name=raidB --dev=deviceB""")

        self.assert_parse_error("""
dmraid --name=raidA --dev=deviceA
dmraid --name=raidA --dev=deviceA""", KickstartParseWarning)

class F24_TestCase(FC6_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("dmraid")
        self.assertEqual(issubclass(parser.__class__, DeprecatedCommand), True)
        parser = parser._getParser()
        self.assertIsNotNone(parser)
        self.assertTrue(parser.description.find('deprecated:: Fedora24') > -1)

class F34_TestCase(F24_TestCase):
    def runTest(self):
        # make sure that dmraid is removed
        cmd = self.handler().commands["dmraid"]
        self.assertTrue(issubclass(cmd.__class__, RemovedCommand))

if __name__ == "__main__":
    unittest.main()



#
# Tomas Radej <tradej@redhat.com>
#
# Copyright 2015-2020 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import gettext
import os
import six

def _find_locale_files():
    module_path = os.path.abspath(__file__)
    locale_path = os.path.join(os.path.dirname(module_path), 'locale')

    gettext.bindtextdomain("pykickstart", locale_path)
    gettext.textdomain("pykickstart")
_find_locale_files()

if six.PY3:
    _ = lambda x: gettext.gettext(x) if x else ''
else:
    _ = lambda x: gettext.lgettext(x) if x else ''



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007-2014 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import RHEL5, RHEL6, versionToLongString
from pykickstart.version import FC3, FC4, F8, F12, F14, F15, F17, F18, F19, F21, F29, F34
from pykickstart.base import KickstartCommand, RemovedCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser, commaSplit

from pykickstart.i18n import _

class FC3_Bootloader(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.driveorder = kwargs.get("driveorder", [])
        self.appendLine = kwargs.get("appendLine", "")
        self.forceLBA = kwargs.get("forceLBA", False)
        self.linear = kwargs.get("linear", True)
        self.location = kwargs.get("location", "none")
        self.md5pass = kwargs.get("md5pass", "")
        self.password = kwargs.get("password", "")
        self.upgrade = kwargs.get("upgrade", False)
        self.useLilo = kwargs.get("useLilo", False)

        self.deleteRemovedAttrs()

    def _getArgsAsStr(self):
        retval = ""

        if self.appendLine:
            retval += " --append=\"%s\"" % self.appendLine
        if self.linear:
            retval += " --linear"
        if self.location:
            retval += " --location=%s" % self.location
        if hasattr(self, "forceLBA") and self.forceLBA:
            retval += " --lba32"
        if self.password:
            retval += " --password=\"%s\"" % self.password
        if self.md5pass:
            retval += " --md5pass=\"%s\"" % self.md5pass
        if self.upgrade:
            retval += " --upgrade"
        if self.useLilo:
            retval += " --useLilo"
        if len(self.driveorder) > 0:
            retval += " --driveorder=\"%s\"" % ",".join(self.driveorder)

        return retval

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.location:
            retval += "# System bootloader configuration\nbootloader"
            retval += self._getArgsAsStr() + "\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="bootloader", description="""
                            This required command specifies how the boot loader
                            should be installed.

                            There must be a biosboot partition for the bootloader
                            to be installed successfully onto a disk that contains
                            a GPT/GUID partition table, which includes disks
                            initialized by anaconda. This partition may be created
                            with the kickstart command
                            ``part biosboot --fstype=biosboot --size=1``. However,
                            in the case that a disk has an existing biosboot
                            partition, adding a ``part biosboot`` option is
                            unnecessary.""", version=FC3)
        op.add_argument("--append", dest="appendLine", version=FC3, help="""
                        Specifies additional kernel parameters. For example:

                        ``bootloader --location=mbr --append="hdd=ide-scsi ide=nodma"``

                        **Note** The installer will add the bootloader arguments ``rhgb
                        quiet`` if plymouth is installed on the target system. You can
                        disable these options with ``-plymouth`` in the ``%%packages``
                        section.
                        """)
        op.add_argument("--linear", action="store_true", default=True,
                        version=FC3, help="use linear mode to access hard disks (for LILO only)")
        op.add_argument("--nolinear", dest="linear", action="store_false",
                        version=FC3, help="do not use linear mode to access hard disks (for LILO only)")
        op.add_argument("--location", default="mbr", version=FC3,
                        choices=["mbr", "partition", "none", "boot"],
                        help="""
                        Specifies where the boot record is written. Valid values
                        are the following: mbr (the default), partition
                        (installs the boot loader on the first sector of the
                        partition containing the kernel), or none
                        (do not install the boot loader).

                         **Note** `bootloader --location=none` is different from
                         `bootloader --location=none --disabled`.
                         `--location=none` prevents extra installation steps
                         that makes the target machine bootable, e.g. write to
                         MBR on x86 BIOS systems. However, the corresponding RPM
                         packages are still installed, and `--disabled` can be
                         appended to prevent it. `bootloader --disabled` only
                         does not prevent the installation of the bootloader and
                         Anaconda will complain if no other options are
                         provided.
                        """)
        op.add_argument("--lba32", dest="forceLBA", action="store_true",
                        default=False, version=FC3, help="force the use of LBA32 mode for hard disk access (LILO only)")
        op.add_argument("--password", default="", version=FC3, help="""
                        If using GRUB, sets the GRUB boot loader password. This
                        should be used to restrict access to the GRUB shell,
                        where arbitrary kernel options can be passed.""")
        op.add_argument("--md5pass", default="", version=FC3, help="""
                        If using GRUB, similar to ``--password=`` except the
                        password should already be encrypted.""")
        op.add_argument("--upgrade", action="store_true", default=False,
                        version=FC3, help="upgrade the boot loader installed on disk")
        op.add_argument("--useLilo", action="store_true", default=False,
                        version=FC3, help="force the use of LILO")
        op.add_argument("--driveorder", type=commaSplit, version=FC3, help="define the explicit hard disk order the boot loader should use")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)

        if self.currentCmd == "lilo":
            self.useLilo = True

        return self


class FC3_Lilo(FC3_Bootloader):
    """
        This is for backwards compatibility and docs generation.
        Used only in FC3, RHEL3 and RHEL4.
    """
    def _getParser(self):
        op = super(FC3_Lilo, self)._getParser()
        op.prog = "lilo"
        return op

class FC4_Lilo(RemovedCommand, FC3_Lilo):
    def _getParser(self):
        op = FC3_Lilo._getParser(self)
        op.description += "\n\n.. versionremoved:: %s" % versionToLongString(FC4)
        return op


class FC4_Bootloader(FC3_Bootloader):
    removedKeywords = FC3_Bootloader.removedKeywords + ["linear", "useLilo"]
    removedAttrs = FC3_Bootloader.removedAttrs + ["linear", "useLilo"]

    def __init__(self, writePriority=10, *args, **kwargs):
        FC3_Bootloader.__init__(self, writePriority, *args, **kwargs)

    def _getArgsAsStr(self):
        retval = ""
        if self.appendLine:
            retval += " --append=\"%s\"" % self.appendLine
        if self.location:
            retval += " --location=%s" % self.location
        if hasattr(self, "forceLBA") and self.forceLBA:
            retval += " --lba32"
        if self.password:
            retval += " --password=\"%s\"" % self.password
        if self.md5pass:
            retval += " --md5pass=\"%s\"" % self.md5pass
        if self.upgrade:
            retval += " --upgrade"
        if len(self.driveorder) > 0:
            retval += " --driveorder=\"%s\"" % ",".join(self.driveorder)
        return retval

    def _getParser(self):
        op = FC3_Bootloader._getParser(self)
        op.remove_argument("--linear", version=FC4)
        op.remove_argument("--nolinear", version=FC4)
        op.remove_argument("--useLilo", version=FC4)
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F8_Bootloader(FC4_Bootloader):
    removedKeywords = FC4_Bootloader.removedKeywords
    removedAttrs = FC4_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        FC4_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.timeout = kwargs.get("timeout", None)
        self.default = kwargs.get("default", "")

    def _getArgsAsStr(self):
        ret = FC4_Bootloader._getArgsAsStr(self)

        if self.timeout is not None:
            ret += " --timeout=%d" % (self.timeout,)
        if self.default:
            ret += " --default=%s" % (self.default,)

        return ret

    def _getParser(self):
        op = FC4_Bootloader._getParser(self)
        op.add_argument("--timeout", type=int, version=F8, help="""
                        Specify the number of seconds before the bootloader
                        times out and boots the default option.""")
        op.add_argument("--default", version=F8, help="""
                        Sets the default boot image in the bootloader
                        configuration.""")
        return op

class F12_Bootloader(F8_Bootloader):
    removedKeywords = F8_Bootloader.removedKeywords
    removedAttrs = F8_Bootloader.removedAttrs

    def _getParser(self):
        op = F8_Bootloader._getParser(self)
        op.add_argument("--lba32", dest="forceLBA", action="store_true",
                        help="force the use of LBA32 mode for hard disk access (LILO only)",
                        deprecated=F12)
        return op

class F14_Bootloader(F12_Bootloader):
    removedKeywords = F12_Bootloader.removedKeywords + ["forceLBA"]
    removedAttrs = F12_Bootloader.removedKeywords + ["forceLBA"]

    def _getParser(self):
        op = F12_Bootloader._getParser(self)
        op.remove_argument("--lba32", version=F14)
        return op

class F15_Bootloader(F14_Bootloader):
    removedKeywords = F14_Bootloader.removedKeywords
    removedAttrs = F14_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F14_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.isCrypted = kwargs.get("isCrypted", False)

    def _getArgsAsStr(self):
        ret = F14_Bootloader._getArgsAsStr(self)

        if self.isCrypted:
            ret += " --iscrypted"

        return ret

    def _getParser(self):
        op = F14_Bootloader._getParser(self)
        op.add_argument("--iscrypted", dest="isCrypted", action="store_true",
                        default=False, version=F15, help="""
                        If given, the password specified by ``--password=`` is
                        already encrypted and should be passed to the bootloader
                        configuration without additional modification.""")
        op.add_argument("--md5pass", dest="_md5pass", version=F15, help="""
                        If using GRUB, similar to ``--password=`` except the password
                        should already be encrypted.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        # argparse doesn't give us a way to set two things at once, so we need to check if
        # _md5pass was given and if so, set everything now.
        if getattr(ns, "_md5pass", None):
            ns.password = ns._md5pass
            ns.isCrypted = True
            del ns._md5pass

        self.set_to_self(ns)
        return self

class F17_Bootloader(F15_Bootloader):
    removedKeywords = F15_Bootloader.removedKeywords
    removedAttrs = F15_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F15_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.bootDrive = kwargs.get("bootDrive", "")

    def _getArgsAsStr(self):
        ret = F15_Bootloader._getArgsAsStr(self)

        if self.bootDrive:
            ret += " --boot-drive=%s" % self.bootDrive

        return ret

    def _getParser(self):
        op = F15_Bootloader._getParser(self)
        op.add_argument("--boot-drive", dest="bootDrive", default="",
                        version=F17, help="""
                        Specifies which drive the bootloader should be written
                        to and thus, which drive the computer will boot from.""")
        return op

    def parse(self, args):
        retval = F15_Bootloader.parse(self, args)

        if "," in retval.bootDrive:     # pylint: disable=no-member
            raise KickstartParseError(_("--boot-drive accepts only one argument"), lineno=self.lineno)

        return retval

class F18_Bootloader(F17_Bootloader):
    removedKeywords = F17_Bootloader.removedKeywords
    removedAttrs = F17_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F17_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.leavebootorder = kwargs.get("leavebootorder", False)

    def _getArgsAsStr(self):
        ret = F17_Bootloader._getArgsAsStr(self)

        if self.leavebootorder:
            ret += " --leavebootorder"

        return ret

    def _getParser(self):
        op = F17_Bootloader._getParser(self)
        op.add_argument("--leavebootorder", action="store_true", default=False,
                        version=F18, help="""
                        On EFI or ISeries/PSeries machines, this option prevents
                        the installer from making changes to the existing list
                        of bootable images.""")
        return op

class RHEL5_Bootloader(FC4_Bootloader):
    removedKeywords = FC4_Bootloader.removedKeywords
    removedAttrs = FC4_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        FC4_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.hvArgs = kwargs.get("hvArgs", "")

    def _getArgsAsStr(self):
        ret = FC4_Bootloader._getArgsAsStr(self)

        if self.hvArgs:
            ret += " --hvargs=\"%s\"" % (self.hvArgs,)

        return ret

    def _getParser(self):
        op = FC4_Bootloader._getParser(self)
        # todo: this is only in RHEL5 and nowhere else
        # possibly shadowed by the way we implement commands
        # inheritance
        op.add_argument("--hvargs", dest="hvArgs", version=RHEL5, help="specify Xen hypervisor kernel arguments")
        return op

class RHEL6_Bootloader(F12_Bootloader):
    removedKeywords = F12_Bootloader.removedKeywords
    removedAttrs = F12_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F12_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.isCrypted = kwargs.get("isCrypted", False)

    def _getArgsAsStr(self):
        ret = F12_Bootloader._getArgsAsStr(self)

        if self.isCrypted:
            ret += " --iscrypted"

        return ret

    def _getParser(self):
        op = F12_Bootloader._getParser(self)
        op.add_argument("--iscrypted", dest="isCrypted", action="store_true",
                        default=False, version=RHEL6, help="""
                        If given, the password specified by ``--password=`` is
                        already encrypted and should be passed to the bootloader
                        configuration without additional modification.""")
        op.add_argument("--md5pass", dest="_md5pass", version=RHEL6, help="""
                        If using GRUB, similar to ``--password=`` except the
                        password should already be encrypted.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        # argparse doesn't give us a way to set two things at once, so we need to check if
        # _md5pass was given and if so, set everything now.
        if getattr(ns, "_md5pass", None):
            ns.password = ns._md5pass
            ns.isCrypted = True
            del ns._md5pass

        self.set_to_self(ns)
        return self

class F19_Bootloader(F18_Bootloader):
    removedKeywords = F18_Bootloader.removedKeywords
    removedAttrs = F18_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F18_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.extlinux = kwargs.get("extlinux", False)

    def _getArgsAsStr(self):
        ret = F18_Bootloader._getArgsAsStr(self)

        if self.extlinux:
            ret += " --extlinux"

        return ret

    def _getParser(self):
        op = F18_Bootloader._getParser(self)
        op.add_argument("--extlinux", action="store_true", default=False,
                        version=F19, help="""
                        Use the extlinux bootloader instead of GRUB. This option
                        only works on machines that are supported by extlinux.""")
        return op

class F21_Bootloader(F19_Bootloader):
    removedKeywords = F19_Bootloader.removedKeywords
    removedAttrs = F19_Bootloader.removedAttrs

    def __init__(self, writePriority=10, *args, **kwargs):
        F19_Bootloader.__init__(self, writePriority, *args, **kwargs)

        self.disabled = kwargs.get("disabled", False)
        self.nombr = kwargs.get("nombr", False)

    def _getArgsAsStr(self):
        if self.disabled:
            return " --disabled"

        ret = F19_Bootloader._getArgsAsStr(self)
        if self.nombr:
            ret += " --nombr"
        return ret

    def _getParser(self):
        op = F19_Bootloader._getParser(self)
        op.add_argument("--disabled", action="store_true", default=False,
                        version=F21, help="""
                        Do not install the boot loader.

                         **Note** `bootloader --location=none` is different from
                         `bootloader --location=none --disabled`.
                         `--location=none` prevents extra installation steps
                         that makes the target machine bootable, e.g. write to
                         MBR on x86 BIOS systems. However, the corresponding RPM
                         packages are still installed, and `--disabled` can be
                         appended to prevent it. `bootloader --disabled` only
                         does not prevent the installation of the bootloader and
                         Anaconda will complain if no other options are
                         provided.
                        """)
        op.add_argument("--nombr", action="store_true", default=False,
                        version=F21, help="do not install the boot loader to the MBR")
        return op

class RHEL7_Bootloader(F21_Bootloader):
    pass

class F29_Bootloader(F21_Bootloader):
    removedKeywords = F21_Bootloader.removedKeywords
    removedAttrs = F21_Bootloader.removedAttrs

    def _getParser(self):
        op = F21_Bootloader._getParser(self)
        op.add_argument("--upgrade", action="store_true", default=False,
                        deprecated=F29, help="upgrade the boot loader installed on disk")
        return op

class RHEL8_Bootloader(F29_Bootloader):
    pass

class F34_Bootloader(F29_Bootloader):
    removedKeywords = F29_Bootloader.removedKeywords
    removedAttrs = F29_Bootloader.removedAttrs

    def _getParser(self):
        op = F29_Bootloader._getParser(self)
        op.remove_argument("--upgrade", version=F34)
        return op

class RHEL9_Bootloader(F34_Bootloader):
    pass
